//
//  Quotient_AppDelegate.m
//  Quotient
//
//  Created by Andrew Merenbach on 11/21/06.
//  Copyright 2006 Andrew Merenbach. All rights reserved.
//

#import "Quotient_AppDelegate.h"
#import "XQPreferencesWindowController.h"

#import "XQEquationController.h"
#import "SNPatternDetector.h"


@implementation Quotient_AppDelegate

- (id)init {
	if (nil != (self = [super init])) {
		equationController = [[XQEquationController alloc] init];
		[equationController setDelegate:self];
		
		patternDetector = [[SNPatternDetector alloc] init];
		[patternDetector setDelegate:self];

		AM_titleForCalculateQuotientButton = @"Calculate";
		AM_titleForDetectRepetendButton = @"Detect";

		AM_quotientString = @"";
		AM_repetendString = @"";
	}
	return self;
}

- (void)dealloc {
	[equationController release];
	equationController = nil;

	[patternDetector release];
	patternDetector = nil;
	
	[AM_titleForCalculateQuotientButton release];
	AM_titleForCalculateQuotientButton = nil;
	
	[AM_titleForDetectRepetendButton release];
	AM_titleForDetectRepetendButton = nil;
	
	[AM_quotientString release];
	AM_quotientString = nil;
	 
	[AM_repetendString release];
	AM_repetendString = nil;

	[super dealloc];
}

- (void)awakeFromNib {
	//[equationController loadValuesFromDefaults];
	[self loadAllValuesFromDefaults];
	
	[self updateTitleForCalculateQuotientButton:equationController];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication {
	return YES;
}

- (void)applicationWillTerminate:(NSNotification *)aNotification {
	[self saveAllValuesToDefaults];
	//[equationController saveValuesToDefaults];
}

- (BOOL)application:(NSApplication *)sender delegateHandlesKey:(NSString *)key {
	BOOL b = [key isEqualToString:@"divisor"] ||
				[key isEqualToString:@"dividend"] ||
				[key isEqualToString:@"scale"] ||
				[key isEqualToString:@"quotientString"];
	return b;
}

- (void)finishedQuotientCalculation:(id)controller {
	[self setQuotientString:[controller quotientString]];
	[calculationProgressIndicator stopAnimation:nil];
	//[self updateTitleForDetectRepetendButton];
}

- (void)finishedPatternDetection:(id)detector {
	[self setRepetendString:[detector outputString]];
	[detectionProgressIndicator stopAnimation:nil];
	//[self updateTitleForDetectRepetendButton];
}

/* cover methods for scripting */
- (void)calculateQuotient:(id)sender {
	if ([self divisor] != 0) {
		[calculationProgressIndicator startAnimation:nil];
		
		[self setQuotientString:@""];
		
		[equationController setDividend:[self dividend]];
		[equationController setDivisor:[self divisor]];
		[equationController setScale:[self scale]];
		
		[equationController launchThreadForQuotientCalculation:sender];
	} else {
		NSBeep();
	
		NSAlert *alert = [NSAlert alertWithMessageText:@"Divide-By-Zero Error"
				defaultButton:@"Cancel"
				alternateButton:nil
				otherButton:nil
				informativeTextWithFormat:@"You cannot divide by zero."];
		[alert setAlertStyle:NSInformationalAlertStyle];
		[alert beginSheetModalForWindow:mainWindow modalDelegate:nil didEndSelector:NULL contextInfo:nil];
	}
}

- (void)detectRepetend:(id)sender {
	[detectionProgressIndicator startAnimation:nil];

	[self setRepetendString:@""];
	[patternDetector setInputString:[self quotientString]];
	[patternDetector launchThreadForPatternDetection:sender];
	
	//[self updateTitleForDetectRepetendButton];
}

- (NSString *)divide:(NSNumber *)dividend by:(NSNumber *)divisor withScale:(NSNumber *)scale {
	NSString *quotientString = nil;
	
	{
		XQEquationController *eqc = [[XQEquationController alloc] init];
		
		[eqc setDivisor:[self divisor]];
		[eqc setDividend:[self dividend]];
		[eqc setScale:[self scale]];
		
		[eqc calculateQuotient:nil];
		quotientString = [eqc quotientString];
		
		[eqc release];
	}
	
	return quotientString;
}

- (NSString *)detectRepetendInString:(NSString *)aString {
	NSString *pattern = nil;
	
	{
		SNPatternDetector *tempPatternDetector = [[SNPatternDetector alloc] init];
		[tempPatternDetector setInputString:aString];
		
		[tempPatternDetector detectPattern:nil];
		pattern = [tempPatternDetector outputString];
		
		[tempPatternDetector release];
	}
	
	return pattern;
}

/* end cover methods for scripting */

- (IBAction)export:(id)sender
{
	NSSavePanel *sp;

	sp = [NSSavePanel savePanel];

	[sp setCanSelectHiddenExtension:YES];
	[sp setRequiredFileType:@"txt"];
	
	[sp
		beginSheetForDirectory:nil
		file:@"Quotient Output.txt"
		modalForWindow:mainWindow
		modalDelegate:self
		didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		NSMutableString *summary = [[NSMutableString alloc] initWithString:@"Quotient Output\n"];
			[summary appendString:@"\n"];
			[summary appendFormat:@"Dividend: %i\n", [equationController dividend]];
			[summary appendFormat:@"Divisor: %i\n", [equationController divisor]];
			[summary appendFormat:@"Scale: %i\n", [equationController scale]];
			[summary appendString:@"\n"];
			[summary appendFormat:@"Quotient:\n%@...\n", [self quotientString]];
			[summary appendString:@"\n"];
			[summary appendFormat:@"Repetend: %@", [self repetendString]];
		[summary writeToFile:[sheet filename] atomically:YES];
		[summary release];
	}
}

- (void)updateTitleForCalculateQuotientButton:(id)controller {
	NSString *newTitle = (![controller isCalculating] ? @"Calculate" : @"Stop");
	[self setTitleForCalculateQuotientButton:newTitle];
}

- (void)updateTitleForDetectRepetendButton:(id)detector {
	NSString *newTitle = (![detector isDetecting] ? @"Detect" : @"Stop");
	[self setTitleForDetectRepetendButton:newTitle];
}

- (void)loadAllValuesFromDefaults {
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	[self setDividend:[userDefaults integerForKey:XQPreferencesDividendKey]];
	[self setDivisor:[userDefaults integerForKey:XQPreferencesDivisorKey]];
	[self setScale:[userDefaults integerForKey:XQPreferencesScaleKey]];
}

- (void)saveAllValuesToDefaults {
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	[userDefaults setInteger:[self dividend] forKey:XQPreferencesDividendKey];
	[userDefaults setInteger:[self divisor] forKey:XQPreferencesDivisorKey];
	[userDefaults setInteger:[self scale] forKey:XQPreferencesScaleKey];
}

/* in case the user enters a blank value */
- (void)setNilValueForKey:(NSString *)key {
	//[self willChangeValueForKey:key];
	[self setValue:[NSNumber numberWithInt:0] forKey:key];
	//[self didChangeValueForKey:key];
}

@end

@implementation Quotient_AppDelegate (Accessors)

- (unsigned)dividend {
	return AM_dividend;
}

- (void)setDividend:(unsigned)anInt {
	AM_dividend = anInt;
}

- (unsigned)divisor {
	return AM_divisor;
}

- (void)setDivisor:(unsigned)anInt {
	AM_divisor = anInt;
}

- (unsigned)scale {
	return AM_scale;
}

- (void)setScale:(unsigned)anInt {
	AM_scale = anInt;
}

/*40 q = int(d/p): REM truncate q to an integer<br/>
50 print q;".";: REM print q, then the decimal point<br/>
60 d = 10*(d mod p): REM set d to 10 times the modulus (remainder) of d divided by p<br/>   -- = 10 * (dividend - (quotient * divisor))
70 q = int(d/p): REM set q to the digit right after the decimal point<br/>
80 print q;: REM print q<br/>
90 if not button and inkey$<>" " then goto 60: REM click mouse or press [SPACE] to interrupt
100 end: REM end the program*/
//}

- (NSString *)titleForCalculateQuotientButton {
	return [[AM_titleForCalculateQuotientButton retain] autorelease];
}

- (void)setTitleForCalculateQuotientButton:(NSString *)aString {
	@synchronized(AM_titleForCalculateQuotientButton) {
		if (aString != AM_titleForCalculateQuotientButton) {
			[AM_titleForCalculateQuotientButton release];
			AM_titleForCalculateQuotientButton = [aString retain];
		}
	}
}

- (NSString *)titleForDetectRepetendButton {
	return [[AM_titleForDetectRepetendButton retain] autorelease];
}

- (void)setTitleForDetectRepetendButton:(NSString *)aString {
	@synchronized(AM_titleForDetectRepetendButton) {
		if (aString != AM_titleForDetectRepetendButton) {
			[AM_titleForDetectRepetendButton release];
			AM_titleForDetectRepetendButton = [aString retain];
		}
	}
}

- (NSString *)quotientString {
	return [[AM_quotientString retain] autorelease];
}

- (void)setQuotientString:(NSString *)aString {
	@synchronized(AM_quotientString) {
		if (aString != AM_quotientString) {
			[AM_quotientString release];
			AM_quotientString = [aString copy];
		}
	}
}

- (NSString *)repetendString {
	return [[AM_repetendString retain] autorelease];
}

- (void)setRepetendString:(NSString *)aString {
	@synchronized(AM_repetendString) {
		if (aString != AM_repetendString) {
			[AM_repetendString release];
			AM_repetendString = [aString copy];
		}
	}
}

@end
