//
//  SNAdditions.m
//  Sniffer
//
//  Created by Andrew Merenbach on 7/11/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "SNAdditions.h"
#import "SNPatternDetector.h"


@implementation NSString (PatternDetection)

- (NSString *)findPattern {
	NSString *pattern = nil;
	unsigned len = [self length];
	
	if (len > 1) {
		NSMutableString *searchString;
		NSString *searchTerm;
		NSArray *groupSizes = [[NSNumber numberWithUnsignedInt:len] factors];
		
		NSEnumerator *e = [groupSizes objectEnumerator];
		id groupSize;
		
		while (groupSize = [e nextObject]) {
			searchTerm = [self substringToIndex:[groupSize unsignedIntValue]];
			
			searchString = [self mutableCopy];
			
			[searchString replaceOccurrencesOfString:searchTerm
				withString:@""
				options:NSLiteralSearch
				range:NSMakeRange(0, len)];
			
			if ([searchString length] == 0) {
				pattern = searchTerm;
				break;
			}
			
			[searchString release];
		}
	}
	
	return pattern;
}

- (NSString *)findPatternWithDelegate:(id)delegate {
	NSString *pattern = nil;
	unsigned len = [self length];
	
	if (len > 1) {
		NSMutableString *searchString;
		NSString *searchTerm;
		NSArray *groupSizes = [[NSNumber numberWithUnsignedInt:len] factors];
		
		NSEnumerator *e = [groupSizes objectEnumerator];
		id groupSize;
		
		while ([delegate isDetecting] && (groupSize = [e nextObject])) {
			searchTerm = [self substringToIndex:[groupSize unsignedIntValue]];
			
			searchString = [self mutableCopy];
			
			[searchString replaceOccurrencesOfString:searchTerm
				withString:@""
				options:NSLiteralSearch
				range:NSMakeRange(0, len)];
			
			if ([searchString length] == 0) {
				pattern = searchTerm;
				break;
			}
			
			[searchString release];
		}
	}
	
	return pattern;
}

- (NSString *)findPatternFromLeft {
	NSString *pattern = [NSString string];
	NSString *substring;
	unsigned idx;
	
	for (idx = [self length]; idx > 0; --idx) {
		substring = [self substringToIndex:idx];
		if (nil != (pattern = [substring findPattern])) {
			break;
		}
	}
	
	return pattern;
}

- (NSString *)findPatternFromRight {
	NSString *pattern = [NSString string];
	NSString *substring;
	unsigned idx;
	unsigned len = [self length];
	
	for (idx = 0; idx < len; ++idx) {
		substring = [self substringFromIndex:idx];
		if (nil != (pattern = [substring findPattern])) {
			break;
		}
	}
	
	return pattern;
}


- (NSString *)findPatternFromRightWithDelegate:(id)delegate {
	NSString *pattern = [NSString string];
	NSString *substring;
	unsigned idx;
	unsigned len = [self length];
	
	for (idx = 0; [delegate isDetecting] && idx < len; ++idx) {
		substring = [self substringFromIndex:idx];
		if (nil != (pattern = [substring findPatternWithDelegate:delegate])) {
			break;
		}
	}
	
	return pattern;
}

- (NSString *)substringPastDecimalPoint {
	NSString *string = [NSString string];

	NSScanner *scanner = [[NSScanner alloc] initWithString:self];
	
	[scanner scanUpToString:@"." intoString:NULL];
	[scanner scanString:@"." intoString:NULL];
	
	unsigned loc = [scanner scanLocation];
	if (loc < [self length]) {
		string = [self substringFromIndex:loc];
	}
	
	[scanner release];
	
	return string;
}

@end


@implementation NSNumber (AMAdditions)

/* works only on positive integers */
- (NSArray *)factors {
	unsigned number = [self unsignedIntValue];

	NSMutableArray *factors = [NSMutableArray arrayWithObject:[NSNumber numberWithUnsignedInt:1]];
	double root = sqrt(number);
	unsigned root_rounded = (unsigned)floor(root);
	double quotient;
	unsigned i;
	
	for (i = 2; i < root_rounded; ++i) {
		quotient = number / (double)i;
		if (floor(quotient) == quotient) {
			[factors addObject:[NSNumber numberWithUnsignedInt:(unsigned)quotient]];
			[factors addObject:[NSNumber numberWithUnsignedInt:(unsigned)i]];
		}
	}
		
	// add square root if it is an integer factor
	if (root_rounded == root) {
		[factors addObject:[NSNumber numberWithUnsignedInt:(unsigned)root]];
	}
	
	// sort ascending
	[factors sortUsingSelector:@selector(compare:)];
	
	return factors;
}

@end
