//
//  SNPatternDetector.m
//  Sniffer
//
//  Created by Andrew Merenbach on 7/11/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "SNPatternDetector.h"
#import "SNAdditions.h"

#import "XQEquationController.h"


@implementation SNPatternDetector

- (id)init {
	if (nil != (self = [super init])) {
		AM_inputString = [[NSString alloc] init];
		AM_outputString = [[NSString alloc] init];
		
		AM_delegate = nil;
		
		[self setDetecting:NO];
	}
	return self;
}

- (void)dealloc {
	[AM_inputString release], AM_inputString = nil;
	[AM_outputString release], AM_outputString = nil;
	[AM_delegate release], AM_delegate = nil;
	
	[super dealloc];
}

- (void)launchThreadForPatternDetection:(id)sender {
	if (![self isDetecting]) {
		[NSThread detachNewThreadSelector:@selector(detectPattern:)
			toTarget:self
			withObject:sender];
	} else {
		[self setDetecting:NO];
	}
}

- (void)detectPattern:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		
	if (![self isDetecting]) {	// sanity-check
		[self setDetecting:YES];
		
		[[self delegate] performSelectorOnMainThread:@selector(updateTitleForDetectRepetendButton:)
			withObject:self
			waitUntilDone:YES];
		
		NSString *inputString = [self inputString];
		NSString *outputString = @"";
		
		inputString = [inputString substringPastDecimalPoint];
		outputString = [inputString findPatternFromRightWithDelegate:self];	// [TODO] add :sender; then::: while ([sender shouldContinueDetection])...
		
		[self performSelectorOnMainThread:@selector(setOutputString:)
			withObject:outputString
			waitUntilDone:YES];
		
		/* [TODO] will this cause a potential race condition? */
		{
			id delegate = [self delegate];
			SEL finishedSelector = @selector(finishedPatternDetection:);
			if ([delegate respondsToSelector:finishedSelector]) {
				[delegate performSelectorOnMainThread:finishedSelector
					withObject:self
					waitUntilDone:YES];
			}
		}
		
		[self setDetecting:NO];
		
		[[self delegate] performSelectorOnMainThread:@selector(updateTitleForDetectRepetendButton:)
			withObject:self
			waitUntilDone:YES];
	}
		
	[pool release];
}

@end

@implementation SNPatternDetector (Accessors)

- (NSString *)inputString {
	return [[AM_inputString retain] autorelease];
}

- (void)setInputString:(NSString *)aString {
	@synchronized(AM_inputString) {
		if (aString != AM_inputString) {
			[AM_inputString release];
			AM_inputString = [aString retain];
		}
	}
}

- (NSString *)outputString {
	return [[AM_outputString retain] autorelease];
}

- (void)setOutputString:(NSString *)aString {
	@synchronized(AM_outputString) {
		if (aString != AM_outputString) {
			[AM_outputString release];
			AM_outputString = [aString retain];
		}
	}
}

- (id)delegate {
	return [[AM_delegate retain] autorelease];
}

- (void)setDelegate:(id)anObject {
	@synchronized(AM_delegate) {
		if (anObject != AM_delegate) {
			[AM_delegate release];
			AM_delegate = [anObject retain];
		}
	}
}

- (BOOL)isDetecting {
	return AM_detecting;
}

- (void)setDetecting:(BOOL)flag {
	AM_detecting = flag;
}

@end
