//
//  XQEquationController.m
//  Quotient
//
//  Created by Andrew Merenbach on 5/11/06.
//  Copyright 2006 Andrew Merenbach. All rights reserved.
//

#import "XQEquationController.h"
//#import "XQPreferencesWindowController.h"


@implementation XQEquationController

- (id)init {
	if (self = [super init]) {
		AM_dividend = 0;
		AM_divisor = 0;
		AM_scale = 0;
		
		AM_quotientString = [[NSString alloc] init];
		
		AM_calculating = NO;
		
		AM_delegate = nil;
		
	}
	return self;
}

- (void)dealloc {
	[AM_quotientString release];
	AM_quotientString = nil;
	
	[AM_delegate release];
	AM_delegate = nil;
	
	[super dealloc];
}

/*- (void)loadValuesFromDefaults {
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	[self setDividend:[userDefaults integerForKey:XQPreferencesDividendKey]];
	[self setDivisor:[userDefaults integerForKey:XQPreferencesDivisorKey]];
	[self setScale:[userDefaults integerForKey:XQPreferencesScaleKey]];
}*/

/*- (void)saveValuesToDefaults {
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	[userDefaults setInteger:[self dividend] forKey:XQPreferencesDividendKey];
	[userDefaults setInteger:[self divisor] forKey:XQPreferencesDivisorKey];
	[userDefaults setInteger:[self scale] forKey:XQPreferencesScaleKey];
}*/

- (void)launchThreadForQuotientCalculation:(id)sender {
	if (![self isCalculating]) {
		[NSThread detachNewThreadSelector:@selector(calculateQuotient:)
			toTarget:self
			withObject:sender];
	} else {
		[self setCalculating:NO];
	}
}

- (void)calculateQuotient:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	[self setCalculating:YES];
	[[self delegate] performSelectorOnMainThread:@selector(updateTitleForCalculateQuotientButton:)
			withObject:self
			waitUntilDone:YES];

	NSMutableString *quotient = [NSMutableString string];

	unsigned b = [self dividend];
	unsigned a = [self divisor];
	unsigned scale = [self scale];

	unsigned q, n;
	
	if (a > 0) {	// sanity-check; cannot divide by zero
	
		q = floor(b / a);
		[quotient appendFormat:@"%i.", q];
		//b = 10 * fmod(b, a);

		for (n = 0; n < scale; ++n) {
			b = 10 * (b - q * a);
			q = floor(b / a);
			[quotient appendFormat:@"%i", q];
			//b = 10 * fmod(b, a);
			//b = 10 * b % a;
		}
	}

		[self performSelectorOnMainThread:@selector(setQuotientString:)
			withObject:quotient
			waitUntilDone:YES];
		
		/* [TODO] will this cause a potential race condition? */
		{
			id delegate = [self delegate];
			SEL finishedSelector = @selector(finishedQuotientCalculation:);
			if ([delegate respondsToSelector:finishedSelector]) {
				[delegate performSelectorOnMainThread:finishedSelector
					withObject:self
					waitUntilDone:YES];
			}
		}
	
	[self setCalculating:NO];
	[[self delegate] performSelectorOnMainThread:@selector(updateTitleForCalculateQuotientButton:)
			withObject:self
			waitUntilDone:YES];
	
	[pool release];
}

/*- (void)setNilValueForKey:(NSString *)key {
	[self willChangeValueForKey:key];
	[self setValue:[NSNumber numberWithInt:0] forKey:key];
	[self didChangeValueForKey:key];
}*/

@end


@implementation XQEquationController (EquationAccessors)

- (unsigned)dividend {
	return AM_dividend;
}

- (void)setDividend:(unsigned)anInt {
	AM_dividend = anInt;
}

- (unsigned)divisor {
	return AM_divisor;
}

- (void)setDivisor:(unsigned)anInt {
	AM_divisor = anInt;
}

- (unsigned)scale {
	return AM_scale;
}

- (void)setScale:(unsigned)anInt {
	AM_scale = anInt;
}

- (NSString *)quotientString {
	return [[AM_quotientString retain] autorelease];
}

- (void)setQuotientString:(NSString *)aString {
	@synchronized(AM_quotientString) {
		if (aString != AM_quotientString) {
			[AM_quotientString release];
			AM_quotientString = [aString copy];
		}
	}
}

- (BOOL)isCalculating {
	return AM_calculating;
}

- (void)setCalculating:(BOOL)flag {
	AM_calculating = flag;
}

- (id)delegate {
	return [[AM_delegate retain] autorelease];
}

- (void)setDelegate:(id)anObject {
	@synchronized(AM_delegate) {
		if (anObject != AM_delegate) {
			[AM_delegate release];
			AM_delegate = [anObject retain];
		}
	}
}

@end
